#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-


# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information

#<<<powerscale_nodes>>>
#lnn 1                                         <- new node
#internal_ip_address 128.221.254.1
#Disk EMPTY Bay  2
#Disk HEALTHY Bay  E0
#...
#Disk HEALTHY Bay  A2
#Disk L3 Bay  1
#fs 56% /
#fs 0% /keystore
#fs 5% /var
#fs 0% /var/crash
#Fans 9600.000 Fan0_Speed
#Fans 9720.000 Fan1_Speed
#Temps na SLIC0_Temp
#...
#Temps 43.000 Drive_IO0_Temp
#Battery passed result1
#Battery N/A result2
#PSU 2 count
#PSU 0 failures
#Uptime 1456434
#lnn 2                                         <- new node
#internal_ip_address 128.221.254.2
#Disk EMPTY Bay  2


# import api
from .agent_based_api.v1 import (
    Metric,
    register,
    render,
    Result,
    State,
    Service,
)

# factory settings:
powerscale_nodes_factory_settings = {
    'upper_levels_Fans' :  (0, 0), 
    'upper_levels_fs'   :  (75.0, 90.0),
    'upper_levels_Temps':  (60.0, 70.0),
    'unit_Fans'         : ' rpm',
    'unit_fs'           : '%',
    'unit_Temps'        : '°C',
    'ok_states_Battery' : ['passed'],
    'ok_states_Disk'    : ['HEALTHY', 'L3']
}

# parse function
def parse_powerscale_nodes(string_table):

    # we'll return this later
    data              = {}

    three_row_entries = ['Disk', 'fs', 'Fans', 'Temps', 'Battery', 'PSU']
    ignore_values     = ['EMPTY', 'N/A', 'na']

    # loop thru
    for line in string_table:

        # a new node
        if line[0] == 'lnn':
            node       = line[1]
            data[node] = {}
            for what in three_row_entries:
                data[node][what] = []
            continue

        # <type> <value> <identifier>
        if line[0] in three_row_entries:
            entry = {}
            if line[1] not in ignore_values:

                entry['val'] = line[1].strip('-%')
                entry['key'] = ' '.join(line[2:])

                data[node][line[0]].append(entry)

            continue


        # all the rest
        data[node][line[0]] = ' '.join(line[1:])

    return data

# discovery function
def discover_powerscale_nodes(section):

    for node in list(section):
        yield Service( item=node)

# check function
def check_powerscale_nodes(item, params, section):

    # initial
    mysummary    = []
    mydetails    = []

    try:
        data=(section[item])
    except:
        yield Result(state=State.UNKNOWN, summary='not found')
        return

    # handle upper levels check on similar readings
    for mytype in ['Fans', 'fs', 'Temps']:

        # check if existing
        try:
            if len(data[mytype]) == 0:
                mysummary.append('No %s info found(!)' % mytype)
                continue
        except:
            mysummary.append('No %s info found(!)' % mytype)
            continue

        # get the params
        unit = params['unit_%s' % mytype]
        warn, crit = params['upper_levels_%s' % mytype]

        # loop tru
        for entry in list(data[mytype]):

            value = float(entry['val'])
            info  = '%s %s: %d%s' % (mytype, entry['key'], value, unit)

            # any error to summary
            if value > crit and crit > 0:
                mysummary.append('%s(!!) (above %s/%s°C)' % (info, warn, crit))
                continue

            # any warning to summary
            if value > warn and crit > 0:
                mysummary.append('%s(!) (above %s/%s°C)' % (info, warn, crit))
                continue

            # nonerror to details
            mydetails.append(info)


    # handle state check on similar readings
    for mytype in ['Disk', 'Battery']:

        # check if existing
        try:
            if len(data[mytype]) == 0:
                mysummary.append('No %s info found(!)' % mytype)
        except:
            mysummary.append('No %s info found(!)' % mytype)
            continue

        # loop thru
        for entry in list(data[mytype]):

            info = '%s %s: %s' % (mytype, entry['key'], entry['val'])

            # any error to summaray
            if entry['val'] not in params['ok_states_%s' % mytype]:
                mysummary.append('%s(!!)' % info)
            else:
                mydetails.append(info)

    # PSU
    try:
        for entry in list(data['PSU']):
            if entry['key'] == 'count':
                psu_count = int(entry['val'])
                continue
            if entry['key'] == 'failures':
                psu_fail  = int(entry['val'])
                continue
        
        # check for error
        if psu_fail != 0:
            mysummary.append('PSU failure(!!)')
        else:
            mydetails.append('PSU count: %s' % psu_count)

    except:
        mysummary.append('No PSU info found(!)')

    # Uptime
    try:
       uptime = int(data['Uptime'])
       mydetails.append('Uptime: %s' % render.timespan(uptime))
       yield Metric('uptime', uptime)
    except:
       pass



    # get worst state from markers
    mysummary = ', '.join(mysummary) if len(mysummary) > 0 else 'Hardware Status OK'
    mydetails = '\n'.join(mydetails)
    mystate   = State.CRIT if '!!' in mysummary else State.WARN if '!' in mysummary else State.OK

    # this is the end, my friend
    yield Result(state=mystate, summary=mysummary, details=mydetails)
    return

        

# register parse function
register.agent_section(
    name                = "powerscale_nodes",
    parse_function      = parse_powerscale_nodes,
)

# register check
register.check_plugin(
    name                     = "powerscale_nodes",
    service_name             = "PowerScale Node %s",
    discovery_function       = discover_powerscale_nodes,
    check_function           = check_powerscale_nodes,
    check_ruleset_name       = "powerscale_nodes",
    check_default_parameters = powerscale_nodes_factory_settings,
)
