#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information

#<<<powerscale_quotas>>>
#path /ifs/<DIR1>/<SUBDIR1>                      <- new quota
#thresholds#hard 32212254720000
#usage#logical 31225176124943
#usage#physical 63689651750400
#path /ifs/<DIR1>/<SUBDIR2>                      <- new quota
#thresholds#hard 107374182400
#usage#logical 12029959
#usage#physical 40714240
#path /ifs/<DIR2>/<SUBDIR1>                      <- new quota
# ...


# import api
from .agent_based_api.v1 import (
    Metric,
    register,
    render,
    Result,
    State,
    Service,
)

# factory settings:
powerscale_quotas_factory_settings = {
    'upper_levels'   :  (95.0, 99.0),
}

# parse function
def parse_powerscale_quotas(string_table):

    # we'll return this later
    data              = []
    show_summary      = False

    #print(string_table)
    # loop thru
    for line in string_table:

        # summary desired
        if line[0] == 'show_summary':
            show_summary = True
            continue

        # a new node
        if line[0] == 'path':
            entry    = {}
            entry['path'] = line[1]
            entry['show_summary'] = show_summary
            continue

        # threshold
        if 'thresholds#' in line[0]:
            entry['thresholds'] = int(line[1])
            continue

        # usage
        if '#logical' in line[0]:
            entry['usage'] = int(line[1])

            # we can calc percentage here, since threshold is first
            percentage = round(entry['usage'] * 100 / entry['thresholds'], 1)
            entry['percentage'] = percentage
            data.append(entry)
            continue

    # sort reverse by percentage and return
    sortdata = sorted(data, key=lambda k : k['percentage'], reverse=True)
    return sortdata

# discovery function
def discover_powerscale_quotas(section):

    for quota in list(section):
        if quota['show_summary'] == True:
            yield Service(item='Summary', parameters={'expected' : len(section)})
            return
        yield Service( item=quota['path'])

# check function
def check_powerscale_quotas(item, params, section):

    # initial
    mysummary    = []
    mydetails    = []

    warn, crit = params['upper_levels']

    # summary
    if item == 'Summary':

        for quota in list(section):

            # percentage
            p = quota['percentage']

            # tell the others
            info = '%s: %.1f%% used' % (quota['path'], p)

            # crit to summary and details
            if p > crit and crit > 0:
                mysummary.append('%s(!!)' % info)
                mydetails.append('%s(!!)' % info)
                continue
                
            # warn to summary and details
            if p > warn and crit > 0:
                mysummary.append('%s(!)' % info)
                mydetails.append('%s(!)' % info)
                continue

            # OK to details
            mydetails.append('%s' % info)


        # everything is OK
        l = len(mysummary)
        t = len(section)    # total
        if l == 0:
            mysummary.append('%s Quota%s OK' % (t, '' if t == 1 else 's'))

        if l == 1 or l == 2:
            s = len(section) - len(mysummary)
            mysummary.append('%s Quota%s OK' % (s, '' if s == 1 else 's'))

        # reset mysummary if too much issues for readability
        if l > 2:
            m = '(!!)' if '!!' in ' '.join(mysummary) else '(!)'
            mysummary    = []
            mysummary.append('%s (of %s) SmartQuota%s over threshold%s' % (
                              l, 
                              t, 
                              '' if l == 1 else 's', 
                              m))

        # count
        e = params['expected']
        if t != e:
            mysummary.append('%s total Quota%s(!) (expected %s)' % (t, '' if t == 1 else 's', e))


        mydetails = '\n'.join(mydetails)

    # single quota
    else:

        # find my quota in list
        for quota in list(section):
            if quota['path'] == item:
                data = quota
                break

        # check for existence
        try:
            data['path']
        except:
            yield Result(state=State.UNKNOWN, summary='not found')
            return

        # check
        p = quota['percentage']
        s = quota['thresholds']
        u = quota['usage']
        m = '(!!)' if p > crit and crit > 0 else '(!)' if p > warn and crit > 0 else ''
        mysummary.append('%.1f%%%s of %s used' % (p, m, render.bytes(s)))

        # Metric
        yield Metric('fs_used', u, boundaries=(0, s), levels=(s*warn/100, s*crit/100))
                

    # get worst state from markers
    mysummary = ', '.join(mysummary)
    mystate   = State.CRIT if '!!' in mysummary else State.WARN if '!' in mysummary else State.OK

    # this is the end, my friend
    try:
        yield Result(state=mystate, summary=mysummary, details=mydetails)
    except:
        yield Result(state=mystate, summary=mysummary)
    return

        

# register parse function
register.agent_section(
    name                = "powerscale_quotas",
    parse_function      = parse_powerscale_quotas,
)

# register check
register.check_plugin(
    name                     = "powerscale_quotas",
    service_name             = "SmartQuota %s",
    discovery_function       = discover_powerscale_quotas,
    check_function           = check_powerscale_quotas,
    check_ruleset_name       = "powerscale_quotas",
    check_default_parameters = powerscale_quotas_factory_settings,
)
